@@ -1,11 +1,11 @@
 package edit
 
 import (
-	"github.com/elves/elvish/pkg/cli"
-	"github.com/elves/elvish/pkg/eval"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/xiaq/persistent/hashmap"
+	"src.elv.sh/pkg/cli"
+	"src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/persistent/hashmap"
 )
 
 //elvdoc:var abbr
@@ -111,8 +111,8 @@ func initInsertAPI(appSpec *cli.AppSpec, nt notifier, ev *eval.Evaler, nb eval.N
 	SmallWordAbbrVar := vars.FromPtr(&SmallWordAbbr)
 	appSpec.SmallWordAbbreviations = makeMapIterator(SmallWordAbbrVar)
 
-	binding := newBindingVar(EmptyBindingMap)
-	appSpec.OverlayHandler = newMapBinding(nt, ev, binding)
+	bindingVar := newBindingVar(emptyBindingsMap)
+	appSpec.CodeAreaBindings = newMapBindings(nt, ev, bindingVar)
 
 	quotePaste := newBoolVar(false)
 	appSpec.QuotePaste = func() bool { return quotePaste.GetRaw().(bool) }
@@ -125,7 +125,7 @@ func initInsertAPI(appSpec *cli.AppSpec, nt notifier, ev *eval.Evaler, nb eval.N
 	nb.Add("small-word-abbr", SmallWordAbbrVar)
 	nb.AddGoFn("<edit>", "toggle-quote-paste", toggleQuotePaste)
 	nb.AddNs("insert", eval.NsBuilder{
-		"binding":     binding,
+		"binding":     bindingVar,
 		"quote-paste": quotePaste,
 	}.Ns())
 }
