@@ -4,12 +4,14 @@ import (
 	"errors"
 	"fmt"
 	"math"
+	"math/big"
 	"sort"
+	"strconv"
 
-	"github.com/elves/elvish/pkg/eval/errs"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/xiaq/persistent/hashmap"
+	"src.elv.sh/pkg/eval/errs"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/persistent/hashmap"
 )
 
 // Sequence, list and maps.
@@ -139,7 +141,10 @@ func makeMap(input Inputs) (vals.Map, error) {
 // ```
 //
 // Output `$low`, `$low` + `$step`, ..., proceeding as long as smaller than
-// `$high`. If not given, `$low` defaults to 0.
+// `$high` or until overflow. If not given, `$low` defaults to 0. The `$step`
+// must be positive.
+//
+// This command is [exactness-preserving](#exactness-preserving).
 //
 // Examples:
 //
@@ -155,44 +160,121 @@ func makeMap(input Inputs) (vals.Map, error) {
 // ▶ 5
 // ```
 //
-// Beware floating point oddities:
+// When using floating-point numbers, beware that numerical errors can result in
+// an incorrect number of outputs:
 //
 // ```elvish-transcript
-// ~> range 0 0.8 &step=.1
-// ▶ 0
-// ▶ 0.1
-// ▶ 0.2
-// ▶ 0.30000000000000004
-// ▶ 0.4
-// ▶ 0.5
-// ▶ 0.6
-// ▶ 0.7
-// ▶ 0.7999999999999999
+// ~> range 0.9 &step=0.3
+// ▶ (num 0.0)
+// ▶ (num 0.3)
+// ▶ (num 0.6)
+// ▶ (num 0.8999999999999999)
+// ```
+//
+// Avoid this problem by using exact rationals:
+//
+// ```elvish-transcript
+// ~> range 9/10 &step=3/10
+// ▶ (num 0)
+// ▶ (num 3/10)
+// ▶ (num 3/5)
 // ```
 //
 // Etymology:
 // [Python](https://docs.python.org/3/library/functions.html#func-range).
 
-type rangeOpts struct{ Step float64 }
+type rangeOpts struct{ Step vals.Num }
 
 func (o *rangeOpts) SetDefaultOptions() { o.Step = 1 }
 
-func rangeFn(fm *Frame, opts rangeOpts, args ...float64) error {
-	var lower, upper float64
-
+func rangeFn(fm *Frame, opts rangeOpts, args ...vals.Num) error {
+	var rawNums []vals.Num
 	switch len(args) {
 	case 1:
-		upper = args[0]
+		rawNums = []vals.Num{0, args[0], opts.Step}
 	case 2:
-		lower, upper = args[0], args[1]
+		rawNums = []vals.Num{args[0], args[1], opts.Step}
 	default:
-		return ErrArgs
+		return errs.ArityMismatch{What: "arguments", ValidLow: 1, ValidHigh: 2, Actual: len(args)}
+	}
+	switch step := opts.Step.(type) {
+	case int:
+		if step <= 0 {
+			return errs.BadValue{
+				What: "step", Valid: "positive", Actual: strconv.Itoa(step)}
+		}
+	case *big.Int:
+		if step.Sign() <= 0 {
+			return errs.BadValue{
+				What: "step", Valid: "positive", Actual: step.String()}
+		}
+	case *big.Rat:
+		if step.Sign() <= 0 {
+			return errs.BadValue{
+				What: "step", Valid: "positive", Actual: step.String()}
+		}
+	case float64:
+		if step <= 0 {
+			return errs.BadValue{
+				What: "step", Valid: "positive", Actual: vals.ToString(step)}
+		}
+	}
+	nums := vals.UnifyNums(rawNums, vals.Int)
+
+	out := fm.ValueOutput()
+	switch nums := nums.(type) {
+	case []int:
+		lower, upper, step := nums[0], nums[1], nums[2]
+		for cur := lower; cur < upper; cur += step {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			if cur+step <= cur {
+				// Overflow
+				break
+			}
+		}
+	case []*big.Int:
+		lower, upper, step := nums[0], nums[1], nums[2]
+		cur := &big.Int{}
+		for cur.Set(lower); cur.Cmp(upper) < 0; {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			next := &big.Int{}
+			next.Add(cur, step)
+			cur = next
+		}
+	case []*big.Rat:
+		lower, upper, step := nums[0], nums[1], nums[2]
+		cur := &big.Rat{}
+		for cur.Set(lower); cur.Cmp(upper) < 0; {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			next := &big.Rat{}
+			next.Add(cur, step)
+			cur = next
+		}
+	case []float64:
+		lower, upper, step := nums[0], nums[1], nums[2]
+		for cur := lower; cur < upper; cur += step {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			if cur+step <= cur {
+				// Overflow
+				break
+			}
+		}
+	default:
+		panic("unreachable")
 	}
 
-	out := fm.OutputChan()
-	for f := lower; f < upper; f += opts.Step {
-		out <- vals.FromGo(f)
-	}
 	return nil
 }
 
@@ -215,11 +297,15 @@ func rangeFn(fm *Frame, opts rangeOpts, args ...float64) error {
 //
 // Etymology: [Clojure](https://clojuredocs.org/clojure.core/repeat).
 
-func repeat(fm *Frame, n int, v interface{}) {
-	out := fm.OutputChan()
+func repeat(fm *Frame, n int, v interface{}) error {
+	out := fm.ValueOutput()
 	for i := 0; i < n; i++ {
-		out <- v
+		err := out.Put(v)
+		if err != nil {
+			return err
+		}
 	}
+	return nil
 }
 
 //elvdoc:fn assoc
@@ -322,9 +408,16 @@ func dissoc(a, k interface{}) (interface{}, error) {
 //
 // @cf one
 
-func all(fm *Frame, inputs Inputs) {
-	out := fm.OutputChan()
-	inputs(func(v interface{}) { out <- v })
+func all(fm *Frame, inputs Inputs) error {
+	out := fm.ValueOutput()
+	var errOut error
+	inputs(func(v interface{}) {
+		if errOut != nil {
+			return
+		}
+		errOut = out.Put(v)
+	})
+	return errOut
 }
 
 //elvdoc:fn one
@@ -351,8 +444,7 @@ func one(fm *Frame, inputs Inputs) error {
 		n++
 	})
 	if n == 1 {
-		fm.OutputChan() <- val
-		return nil
+		return fm.ValueOutput().Put(val)
 	}
 	return fmt.Errorf("expect a single value, got %d", n)
 }
@@ -381,15 +473,20 @@ func one(fm *Frame, inputs Inputs) error {
 //
 // Etymology: Haskell.
 
-func take(fm *Frame, n int, inputs Inputs) {
-	out := fm.OutputChan()
+func take(fm *Frame, n int, inputs Inputs) error {
+	out := fm.ValueOutput()
+	var errOut error
 	i := 0
 	inputs(func(v interface{}) {
+		if errOut != nil {
+			return
+		}
 		if i < n {
-			out <- v
+			errOut = out.Put(v)
 		}
 		i++
 	})
+	return errOut
 }
 
 //elvdoc:fn drop
@@ -419,15 +516,20 @@ func take(fm *Frame, n int, inputs Inputs) {
 //
 // @cf take
 
-func drop(fm *Frame, n int, inputs Inputs) {
-	out := fm.OutputChan()
+func drop(fm *Frame, n int, inputs Inputs) error {
+	out := fm.ValueOutput()
+	var errOut error
 	i := 0
 	inputs(func(v interface{}) {
+		if errOut != nil {
+			return
+		}
 		if i >= n {
-			out <- v
+			errOut = out.Put(v)
 		}
 		i++
 	})
+	return errOut
 }
 
 //elvdoc:fn has-value
@@ -586,8 +688,7 @@ func count(fm *Frame, args ...interface{}) (int, error) {
 	default:
 		// The error matches what would be returned if the `Inputs` API was
 		// used. See GoFn.Call().
-		return 0, errs.ArityMismatch{
-			What: "arguments here", ValidLow: 0, ValidHigh: 1, Actual: nargs}
+		return 0, errs.ArityMismatch{What: "arguments", ValidLow: 0, ValidHigh: 1, Actual: nargs}
 	}
 	return n, nil
 }
@@ -612,11 +713,16 @@ func count(fm *Frame, args ...interface{}) (int, error) {
 // Note that there is no guaranteed order for the keys of a map.
 
 func keys(fm *Frame, v interface{}) error {
-	out := fm.OutputChan()
-	return vals.IterateKeys(v, func(k interface{}) bool {
-		out <- k
-		return true
+	out := fm.ValueOutput()
+	var errPut error
+	errIterate := vals.IterateKeys(v, func(k interface{}) bool {
+		errPut = out.Put(k)
+		return errPut == nil
 	})
+	if errIterate != nil {
+		return errIterate
+	}
+	return errPut
 }
 
 //elvdoc:fn order
@@ -771,8 +877,12 @@ func order(fm *Frame, opts orderOptions, inputs Inputs) error {
 	if errSort != nil {
 		return errSort
 	}
+	out := fm.ValueOutput()
 	for _, v := range values {
-		fm.OutputChan() <- v
+		err := out.Put(v)
+		if err != nil {
+			return err
+		}
 	}
 	return nil
 }
@@ -788,23 +898,21 @@ const (
 
 func compare(a, b interface{}) ordering {
 	switch a := a.(type) {
-	case float64:
-		if b, ok := b.(float64); ok {
-			switch {
-			case math.IsNaN(a):
-				if math.IsNaN(b) {
-					return equal
-				}
-				return less
-			case math.IsNaN(b):
-				return more
-			case a == b:
-				return equal
-			case a < b:
-				return less
+	case int, *big.Int, *big.Rat, float64:
+		switch b.(type) {
+		case int, *big.Int, *big.Rat, float64:
+			a, b := vals.UnifyNums2(a, b, 0)
+			switch a := a.(type) {
+			case int:
+				return compareInt(a, b.(int))
+			case *big.Int:
+				return compareInt(a.Cmp(b.(*big.Int)), 0)
+			case *big.Rat:
+				return compareInt(a.Cmp(b.(*big.Rat)), 0)
+			case float64:
+				return compareFloat(a, b.(float64))
 			default:
-				// a > b
-				return more
+				panic("unreachable")
 			}
 		}
 	case string:
@@ -844,3 +952,32 @@ func compare(a, b interface{}) ordering {
 	}
 	return uncomparable
 }
+
+func compareInt(a, b int) ordering {
+	if a < b {
+		return less
+	} else if a > b {
+		return more
+	}
+	return equal
+}
+
+func compareFloat(a, b float64) ordering {
+	// For the sake of ordering, NaN's are considered equal to each
+	// other and smaller than all numbers
+	switch {
+	case math.IsNaN(a):
+		if math.IsNaN(b) {
+			return equal
+		}
+		return less
+	case math.IsNaN(b):
+		return more
+	case a < b:
+		return less
+	case a > b:
+		return more
+	default: // a == b
+		return equal
+	}
+}
