@@ -3,17 +3,16 @@ package eval_test
 import (
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval"
-	"github.com/elves/elvish/pkg/testutil"
+	. "src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/testutil"
 
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/elves/elvish/pkg/parse"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/parse"
 )
 
 func TestPurelyEvalCompound(t *testing.T) {
-	home, cleanup := testutil.InTempHome()
-	defer cleanup()
+	home := testutil.InTempHome(t)
 
 	var tests = []struct {
 		code      string
@@ -53,7 +52,7 @@ func TestPurelyEvalCompound(t *testing.T) {
 		t.Run(test.code, func(t *testing.T) {
 			n := &parse.Compound{}
 			err := parse.ParseAs(
-				parse.Source{Name: "[test]", Code: test.code}, n, nil)
+				parse.Source{Name: "[test]", Code: test.code}, n, parse.Config{})
 			if err != nil {
 				panic(err)
 			}
